/* document-message.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

/**
 * A message widget for Valide.AbstractTextDocument
 */
public class Valide.DocumentMessage : Gtk.HBox
{
  private Gtk.Label primary_label;
  private Gtk.Label secondary_label;

  /**
   * The attached document
   */
  public AbstractTextDocument document
  {
    construct;
    get;
  }

  /**
   * Create a new DocumentMessage
   *
   * @param document The attached document
   */
  public DocumentMessage (AbstractTextDocument document)
  {
    this.document = document;
  }

  construct
  {
    Gtk.Image image;
    Gtk.VBox vbox;
    Gtk.VButtonBox box;
    Gtk.Button button;
    string text;

    this.set_spacing (16);
    this.set_border_width (8);

    image = new Gtk.Image.from_stock (Gtk.STOCK_DIALOG_WARNING,
                                      Gtk.IconSize.DIALOG);
    this.pack_start (image, false, true, 0);


    vbox = new Gtk.VBox (false, 0);
    this.pack_start (vbox, true, true, 0);

    this.primary_label = new Gtk.Label ("");
    this.primary_label.set_alignment (0, 0.5f);
    this.primary_label.set_justify (Gtk.Justification.LEFT);
    vbox.pack_start (this.primary_label, true, true, 0);

    this.secondary_label = new Gtk.Label ("");
    this.secondary_label.set_alignment (0, 0.5f);
    this.secondary_label.set_justify (Gtk.Justification.LEFT);
    vbox.pack_start (this.secondary_label, false, true, 0);

    box = new Gtk.VButtonBox ();
    this.pack_start (box, false, true, 0);

    button = new Gtk.Button.from_stock (Gtk.STOCK_REFRESH);
    box.pack_start (button, true, true, 0);
    button.clicked.connect (() => {
      this.document.reload ();
      this.document.state = DocumentState.NORMAL;
      this.document.grab_focus ();
      this.destroy ();
    });

    button = new Gtk.Button.from_stock (Gtk.STOCK_CANCEL);
    box.pack_start (button, true, true, 0);
    button.clicked.connect (() => {
      this.document.is_save = false;
      this.document.state = DocumentState.NORMAL;
      this.document.grab_focus ();
      this.destroy ();
    });

    switch (this.document.state)
    {
      case DocumentState.EXTERNALLY_MODIFIED:
        text = _("The file %s changed on disk.").printf (this.document.path);
        this.primary_label.set_text (text);

        if (!this.document.is_save)
        {
	        text = _("Do you want to drop your changes and reload the file?");
	      }
        else
        {
	        text = _("Do you want to reload the file?");
	      }
        this.secondary_label.set_text (text);
      break;
      case DocumentState.BAD_ENCODING:
        text = _("Couldn't open the file %s.").printf (this.document.path);
        this.primary_label.set_text (text);

        text="""Val(a)IDE has not been able to detect the character coding.
Please check that you are not trying to open a binary file.""";
        this.secondary_label.set_text (text);
      break;
    }
  }
}

