/* executable.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

/**
 * A class representing an program can launch by #ExecutableManager.
 */
public class Valide.Executable : Object
{
  /**
   * A message show when executable was launched.
   *
   * You can use $cmd for show the command.
   */
  protected virtual string start_msg
  {
    owned get
    {
      return "";
    }
  }

  /**
   * A message show when executable stop.
   *
   * You can use :
   *   - $status the exit status of the command.
   *   - $time the execution time 
   */
  protected virtual string end_msg
  {
    owned get
    {
      return _("<b>Process return {$status}  execution time: {$time} s</b>");
    }
  }

  /**
   * Executable current working directory, or NULL to inherit parent's.
   */
  public virtual string working_directory
  {
    owned get
    {
      return "";
    }
  }

  /**
   * An executable name.
   */
  public virtual string executable
  {
    owned get;
    set;
  }

  /**
   * This function is called when executable is launched
   *
   * @param cmd The command line
   *
   * @return A text
   */
  public string cmd_start (string cmd)
  {
    string msg;

    msg = this.start_msg;
    msg = msg.replace ("{$cmd}", cmd);
    return msg;
  }

  /**
   * This function is called when executable quit
   *
   * @param status The exit status
   * @param time The enlapsed time
   *
   * @return A text
   */
  public string cmd_end (int status, double time)
  {
    string msg;

    msg = this.end_msg;
    msg = msg.replace ("{$status}", status.to_string ());
    msg = msg.replace ("{$time}", time.to_string ());
    return msg;
  }

  /**
   * This function was called before adding line in ExecutableManager tree view.
   * You can modify it.
   *
   * @param line the text.
   */
  protected virtual Gdk.Color get_color_line (string line)
  {
    Gdk.Color color;

    Gdk.Color.parse ("#000000", out color);
    return color;
  }

  /**
   * This function was called when line in ExecutableManager was activated.
   *
   * @param line the line was actived.
   */
  protected virtual void line_activated (string line)
  {
  }
}

