/* markup.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *  Nicolas Joseph <nicolas.joseph@valaide.org>
 */

/**
 * Class for manipulate XML files
 */
public class Valide.Markup
{
  private static void save (string filename, Xml.Doc* xml)
  {
    xml->save_file (filename);
  }

  /**
   * Remove an element form a XML file
   *
   * @param filename The path of the XML file
   * @param xpath The xpath of the element
   */
  public static void remove (string filename, string xpath)
  {
    Xml.Doc* xml = Xml.Parser.parse_file (filename);
    Xml.XPathContext xpath_context = new Xml.XPathContext (xml);
    Xml.XPathObject* xpath_obj = xpath_context.eval (xpath);
    for (int i = 0; i < xpath_obj->nodesetval->length (); i++)
    {
      xpath_obj->nodesetval->item (i)->unlink ();
    }
    Markup.save (filename, xml);
  }

  /**
   * Add a children in a XML file
   *
   * @param filename The path of the XML file
   * @param xpath The xpath of the element
   * @param name The name of a children markup
   * @param content The content of the children
   */
  public static void add_child (string filename, string xpath, string name,
                                string? content=null)
  {
    Xml.Doc* xml = Xml.Parser.parse_file (filename);
    Xml.XPathContext xpath_context = new Xml.XPathContext (xml);
    Xml.XPathObject* xpath_obj = xpath_context.eval (xpath);
    xpath_obj->nodesetval->item (0)->new_text_child (null, name, content);
    Markup.save (filename, xml);
  }

  /**
   * Test if an element exist
   *
   * @param filename The path of the XML file
   * @param xpath The xpath of the element
   */
  public static bool exist (string filename, string xpath)
  {
    bool ret = false;

    Xml.Doc* xml = Xml.Parser.parse_file (filename);
    Xml.XPathContext xpath_context = new Xml.XPathContext (xml);
    Xml.XPathObject* xpath_obj = xpath_context.eval (xpath);
    if (xpath_obj != null && xpath_obj->nodesetval->length () > 0)
    {
      ret = true;
    }
    return ret;
  }

  /**
   * Get the content of a markup
   *
   * @param filename The path of the XML file
   * @param xpath The xpath of the element
   *
   * @return The content of the markup
   */
  public static string get_content (string filename, string xpath)
  {
    string ret = null;
    Xml.Doc* xml = Xml.Parser.parse_file (filename);
    Xml.XPathContext xpath_context = new Xml.XPathContext (xml);
    Xml.XPathObject* xpath_obj = xpath_context.eval (xpath);
    if (xpath_obj != null && xpath_obj->nodesetval->length () > 0)
    {
      ret = xpath_obj->nodesetval->item (0)->content;
    }
    return ret;
  }

  /**
   * Get the node
   *
   * @param filename The path of the XML file
   * @param xpath The xpath of the element
   *
   * @return The node class
   */
  public static Xml.NodeSet* get_nodes (string filename, string xpath)
  {
    Xml.Doc* xml = Xml.Parser.parse_file (filename);
    Xml.XPathContext xpath_context = new Xml.XPathContext (xml);
    Xml.XPathObject* xpath_obj = xpath_context.eval (xpath);
    return xpath_obj->nodesetval;
  }
}

