/* source-view.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

/**
 * A widget for show highlighting text
 */
public class Valide.SourceView : Gtk.SourceView
{
  /**
   * The language
   */
  public string language
  {
    get;
    construct;
  }

  /**
   * The buffer
   */
  public new SourceBuffer buffer
  {
    get
    {
      return base.buffer as SourceBuffer;
    }
    set
    {
      base.buffer = value;
    }
  }

  /**
   * The text font
   */
  public string font
  {
    set
    {
      Pango.FontDescription desc;

      desc =  Pango.FontDescription.from_string (value);
      this.modify_font (desc);
    }
  }

  /**
   * The style scheme
   */
  public string style_scheme
  {
    set
    {
      Gtk.SourceStyleScheme scheme;
      Gtk.SourceStyleSchemeManager manager;

      manager = new Gtk.SourceStyleSchemeManager ();
      if (value != "")
      {
        scheme = manager.get_scheme (value);
      }
      else
      
      {
        scheme = manager.get_scheme ("classic");
      }
      this.buffer.set_style_scheme (scheme);
    }
  }

  /**
   * Create a new Valide.SourceView
   *
   * @param language The language for coloration
   */
  public SourceView (string language)
  {
    this.language = language;
  }

  construct
  {
    ConfigManager config;
    Gtk.SourceLanguageManager lm;

    lm = new Gtk.SourceLanguageManager ();
    if (this.language != "txt")
    {
      SourceBuffer buffer;

      buffer = new SourceBuffer ();
      buffer.language = lm.get_language (this.language);
      buffer.set_highlight_syntax (true);
      this.buffer = buffer;
    }
    else
    {
      this.buffer = new SourceBuffer ();
    }

    config = ConfigManager.get_instance ();
    this.set_right_margin_position (config.get_integer ("GtkSourceView", "right-margin-position"));
    this.set_show_right_margin (config.get_boolean ("GtkSourceView", "show-right-margin"));
    this.set_show_line_numbers (config.get_boolean ("GtkSourceView", "show-line-numbers"));
    this.set_auto_indent (config.get_boolean ("GtkSourceView", "auto-indent"));
    this.set_highlight_current_line (config.get_boolean ("GtkSourceView", "highlight-current-line"));
    this.set_indent_on_tab (config.get_boolean ("GtkSourceView", "indent-on-tab"));
    this.set_insert_spaces_instead_of_tabs (config.get_boolean ("GtkSourceView", "insert-spaces-instead-of-tabs"));
    this.set_smart_home_end (Gtk.SourceSmartHomeEndType.BEFORE);
    this.set_tab_width (config.get_integer ("GtkSourceView", "tab-width"));
    this.font = config.get_string ("GtkSourceView", "font");
    this.style_scheme = config.get_string ("GtkSourceView", "style-scheme");
  }

  /**
   * Scroll text view to cursor
   */
  public void scroll_to_cursor ()
  {
    Gtk.TextBuffer buffer;

    buffer = this.buffer;
    this.scroll_to_mark (buffer.get_insert (), 0.25, false, 0.0, 0.0);
  }
}

