/* split-source-view.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

/**
 * A widget with two Valide.SourceView with the same buffer
 */
public class Valide.SplitSourceView : Gtk.VPaned, Completion.Trigger
{
  private weak SourceView last_active;

  /**
   * The language for syntax
   */
  public string language
  {
    get;
    construct;
  }

  /**
   * The currently active text view
   */
  public SourceView active_view
  {
    get
    {
      if (this.get_position () == 0)
      {
        return this.view2;
      }
      else
      {
        return this.last_active;
      }
    }
  }

  /**
   * The first text view (on top)
   */
  public SourceView view1
  {
    get;
    construct;
  }

  /**
   * The first text view (on bottom)
   */
  public SourceView view2
  {
    get;
    construct;
  }

  private bool focus_in_event_cb (Gtk.Widget sender, Gdk.EventFocus event)
  {
    this.last_active = sender as SourceView;
    return false;
  }

  /**
   * Create a new Valide.SplitSourceView
   *
   * @param language The language for coloration
   */
  public SplitSourceView (string language)
  {
    this.language = language;
  }

  construct
  {
    Gtk.ScrolledWindow scrolled_window;


    this.view1 = new SourceView (this.language);
    this.view1.focus_in_event.connect (this.focus_in_event_cb);
    this.view2 = new SourceView (this.language);
    this.view2.focus_in_event.connect (this.focus_in_event_cb);

    this.view2.buffer = this.view1.buffer;

    scrolled_window = new Gtk.ScrolledWindow (null, null);
    scrolled_window.set_policy (Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC);
    this.add1 (scrolled_window);
    scrolled_window.add (this.view1);

    scrolled_window = new Gtk.ScrolledWindow (null, null);
    scrolled_window.set_policy (Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC);
    this.add2 (scrolled_window);
    scrolled_window.add (this.view2);

    this.set_position (0);
    this.last_active = this.view2;
  }
}

