/* heritage.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

public class Valide.Symbols.Heritage : Object
{
  /**
   * The base symbol id
   */
  public int base_id;

  /**
   * The derived symbol full name
   */
  public string derived_name;

  /**
   * @see Valide.Symbols.Object.table_name
   */
  protected override string table_name
  {
    owned get
    {
      return "heritage";
    }
  }

  /**
   * @see Valide.Symbols.Object.table_name
   */
  protected override string add_query
  {
    owned get
    {
      return "INSERT INTO heritage (symbol_id_base, symbol_name_derived) VALUES (?, ?);";
    }
  }

  /**
   * @see Valide.Symbols.Object.exist_query
   */
  protected override string exist_query
  {
    owned get
    {
      return "SELECT symbol_id_base FROM heritage WHERE symbol_id_base = ? AND symbol_name_derived = ?;";
    }
  }

  /**
   * @see Valide.Symbols.Object.bind_exist_stmt
   */
  protected override void bind_exist_stmt ()
  {
    this.exist_stmt.bind_int (1, this.base_id);
    this.exist_stmt.bind_text (1, this.derived_name);
  }

  /**
   * Create a new hetitage
   *
   * @param base_id The base symbol id
   * @param derived_name The derived symbol full name
   */
  public Heritage (int base_id, string derived_name)
  {
    this.base_id = base_id;
    this.derived_name = derived_name;
    this.exist ();
  }

  /**
   * @see Valide.Symbols.Object.add
   */
  public override int add () throws EngineError
  {
    if (this.base_id > 0 && this.derived_name != "" && !this.exist ())
    {
      this.add_stmt.bind_int (1, this.base_id);
      this.add_stmt.bind_text (2, this.derived_name);
      if (this.add_stmt.step () == Sqlite.DONE)
      {
        this.add_stmt.reset ();
      }
      else
      {
        throw new EngineError.EXEC (_("Add heritage '%d->%s': %s"),
                                    this.base_id, this.derived_name,
                                    this.engine.errmsg ());
      }
    }
    return -1;
  }

  /**
   * Get the derived symbol for a symbol
   *
   * @param symbol A symbol
   *
   * @return The derived symbol
   */
  public static string[] get_derived (Symbol symbol) throws EngineError
  {
    string sql;
    string[] derived = null;
    Sqlite.Statement statement;

    sql = "SELECT symbol_name_derived FROM heritage WHERE symbol_id_base = ?;";
    statement = engine.prepare (sql);
    statement.bind_int (1, symbol.id);
    while (statement.step () == Sqlite.ROW)
    {
      derived += statement.column_text (0);
    }
    return derived;
  }
}

