/* language.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

public class Valide.Symbols.Language : Object
{
  /**
   * The language name
   */
  public string name;

  /**
   * @see Valide.Symbols.Object.table_name
   */
  protected override string table_name
  {
    owned get
    {
      return "language";
    }
  }

  /**
   * @see Valide.Symbols.Object.table_name
   */
  protected override string add_query
  {
    owned get
    {
      return "INSERT INTO language (name) VALUES (?);";
    }
  }

  /**
   * @see Valide.Symbols.Object.exist_query
   */
  protected override string exist_query
  {
    owned get
    {
      return "SELECT id FROM language WHERE name = ?;";
    }
  }

  /**
   * @see Valide.Symbols.Object.bind_exist_stmt
   */
  protected override void bind_exist_stmt ()
  {
    this.exist_stmt.bind_text (1, this.name);
  }

  /**
   * Create a new language
   *
   * @param name The language name
   */
  public Language (string name)
  {
    this.name = name;
    this.exist ();
  }

  /**
   * Create a new language from id
   *
   * @param id The language id
   */
  public Language.from_id (int id)
  {
    string sql;
    Sqlite.Statement statement;

    sql = """SELECT id, name
               FROM language
               WHERE id = ?;""";
    statement = this.engine.prepare (sql);
    statement.bind_int (1, id);
    if (statement.step () == Sqlite.ROW)
    {
      this.id = statement.column_int (0);
      this.name = statement.column_text (1);
    }
  }

  /**
   * @see Valide.Symbols.Object.add
   */
  public override int add () throws EngineError
  {
    if (this.name != "" && !this.exist ())
    {
      this.add_stmt.bind_text (1, this.name);
      if (this.add_stmt.step () == Sqlite.DONE)
      {
        this.id = this.get_last_id ();
        this.add_stmt.reset ();
      }
      else
      {
          throw new EngineError.EXEC (_("Add language '%s': %s"),
                                    this.name, this.engine.errmsg ());
      }
    }
    return this.id;
  }
}

