/* template.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

/**
 * A class representing a template for project
 */
public class Valide.Template : Object
{
  /**
   * The name of the template
   */
  public string name
  {
    get;
    private set;
  }

  /**
   * The description of the template
   */
  public string desc
  {
    get;
    private set;
  }

  /**
   * The path of the template file
   */
  public string path
  {
    get;
    private set;
  }

  /**
   * The name of the template file
   */
  public string filename
  {
    get;
    construct;
  }

  /**
   * The icon of the template
   */
  public Gdk.Pixbuf icon
  {
    get;
    private set;
  }

  /**
   * The programming language of the template
   */
  public string lang
  {
    get;
    construct;
  }

  /**
   * The list of source files
   */
  public unowned List<string> files
  {
    get;
    private set;
  }

  /**
   * Create a new template from file
   *
   * @param filename The template file
   * @param lang The programming language
   */
  public Template (string filename, string lang)
  {
    this.filename = filename;
    this.lang = lang;
  }

  construct
  {
    try
    {
      Xml.Doc* xml;

      xml = Xml.Parser.parse_file (this.filename);
      if (xml != null)
      {
        string file;
        Xml.XPathContext xpath;
        Xml.XPathObject* xpath_obj;

        this.path = Path.get_dirname (this.filename);
        xpath = new Xml.XPathContext (xml);
        xpath_obj = xpath.eval ("/template/files/file/text ()");

        for (int i = 0; i < xpath_obj->nodesetval->length (); i++)
        {
          file = Path.build_filename (this.path,
                                      xpath_obj->nodesetval->item (i)->content,
                                      null);

          if (this.lang == "genie")
          {
            file = file.replace (".vala", ".gs");
          }
          if (FileUtils.test (file, FileTest.EXISTS))
          {
            this.files.append (file);
          }
        }

        xpath_obj = xpath.eval ("/template/name/text ()");
        this.name = xpath_obj->nodesetval->item (0)->content;

        xpath_obj = xpath.eval ("/template/description/text ()");
        this.desc = xpath_obj->nodesetval->item (0)->content;

        this.icon = new Gdk.Pixbuf.from_file (Path.build_filename (this.path, "/icon.png"));
      }
    }
    catch (Error e)
    {
      debug (e.message);
    }
  }
}

