/* valac.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *  Nicolas Joseph <nicolas.joseph@valaide.org>
 */

/* @fixme Use static properties */
/**
 * Vala compiler informations
 */
public class Valide.Valac
{
  /**
   * The name of the vala compiler
   */
  public string compiler_name
  {
    owned get
    {
      return "valac";
    }
  }

  /**
   * The path of the vala compiler
   */
  public string prefix
  {
    owned get
    {
      string vala_path = Environment.find_program_in_path (this.compiler_name);
      string bin_path = Path.get_dirname (vala_path);

      return Path.get_dirname (bin_path);
    }
  }

  /**
   * The default vapi directory
   */
  public string vapi_dir
  {
    owned get
    {
      string dir;

      if (Config.OS == "win32")
      {
        dir = Path.build_filename ("C:", "Documents and Settings", "All Users", "Application Data", "vala", "vapi");
      }
      else
      {
        dir = Path.build_filename (this.prefix, "share", "vala", "vapi");
      }
      return dir;
    }
  }

  /**
   * Get the list of the available packages
   *
   * @param vapi_dir The additionnal vapi directory
   *
   * @return The package list
   */
  public static List<string> get_package_list (List<string>? vapi_dir = null)
  {
    List<string> vapi_files = null;
    List<string> vapidir = null;

    if (vapi_dir != null)
    {
      foreach (string v in vapi_dir)
      {
        vapidir.append (v);
      }
    }

    /* @bug static property doesn't work */
    Valac vala = new Valac ();
    vapidir.append (vala.vapi_dir);

    foreach (string d in Environment.get_system_data_dirs ())
    {
      if (FileUtils.test (d, FileTest.IS_DIR))
      {
        vapidir.append (d);
      }
    }

    foreach (string d in vapidir)
    {
      try
      {
        string filename;
        Dir dir = Dir.open (d);

        while ((filename = dir.read_name ()) != null)
        {
          if (filename.has_suffix (".vapi"))
          {
            vapi_files.append (filename.substring (0, filename.size () - 5));
          }
        }
      }
      catch (Error e)
      {
        debug (e.message);
      }
    }
    return vapi_files;
  }

  /**
   * Get the filename of a package
   *
   * @param vapi The name of a package
   *
   * @return The filename
   */
  public static string get_vapi_filename (string vapi)
  {
    string vapi_file;
    string filename = null;

    if (vapi.has_suffix (".vapi"))
    {
      vapi_file = vapi;
    }
    else
    {
      vapi_file = "%s.vapi".printf (vapi);
    }

    /* @bug static property doesn't work */
    Valac vala = new Valac ();
    filename = Path.build_filename (vala.vapi_dir, vapi_file);
    if (!FileUtils.test (filename, FileTest.EXISTS))
    {
      filename = null;
    }
    return filename;
  }
}

