/* vala-provider.vala
 *
 * Copyright (C) 2008-2009  Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 *  Nicolas Joseph <nicolas.joseph@valaide.org>
 */

using Valide;

public class DocumentProvider : Valide.Completion.Provider, Object
{
  private int count;
  private List<Valide.Completion.Proposal> data_list;
  private HashTable<string, int> current_words;

  public DocumentManager documents
  {
    get;
    construct;
  }

  public Symbols.Cache symbols
  {
    get;
    construct;
  }

  public AbstractTextDocument? document
  {
    owned get
    {
      iDocument document;

      document = this.documents.current;
      if (document is AbstractTextDocument)
      {
        return document as AbstractTextDocument;
      }
      else
      {
        return null;
      }
    }
  }

  private bool add_symbol (Symbols.Symbol symbol)
  {
    Valide.Completion.Proposal proposal;

    proposal = new Valide.Completion.Proposal (symbol.name, symbol.icon);
    this.data_list.append (proposal);
    this.count++;
    if (this.count < 100)
    {
      return true;
    }
    else
    {
      return false;
    }
  }

  public DocumentProvider (Symbols.Cache symbols, DocumentManager documents)
  {
    this.symbols = symbols;
    this.documents = documents;
    this.current_words = null;
    this.count = 0;
  }

  /**
   * @see Valide.Completion.Proposal
   */
  public unowned List<Valide.Completion.Proposal>? get_proposals (Valide.SourceView source_view)
  {
    Symbols.Symbol type;
    Symbols.Parser parser;

    this.count = 0;
    this.data_list = new List<Valide.Completion.Proposal> ();

    parser = new Symbols.Parser ();
    type = parser.get_current_type (source_view.buffer);
    if (type != null)
    {
      string filter;

      filter = source_view.buffer.get_last_word ();
      this.symbols.get_completion_for_symbol (source_view.buffer.path,
                                              type,
                                              filter,
                                              this.add_symbol);
    }
    this.current_words = null;
    return this.data_list;
  }
}

